import { __awaiter, __decorate } from "tslib";
/**
 * See https://fdc3.finos.org/docs/fdc3-intro, and https://github.com/finos/FDC3/tree/master/src/api
 */
import { addIntentListener, fdc3Ready, findIntent, findIntentsByContext, getCurrentChannel, getOrCreateChannel, joinChannel, raiseIntent, raiseIntentForContext, } from '@finos/fdc3';
import { observable } from '@microsoft/fast-element';
import { DI } from '@microsoft/fast-foundation';
import { ReplaySubject } from 'rxjs';
import { stripOutBigInt } from './util/fdc3-util';
import { logger } from './utils';
/**
 * The FDC3 implementation.
 * @internal
 */
export class DefaultFDC3 {
    constructor() {
        this.isReady = false;
        this._isReady$ = new ReplaySubject();
        this.isReady$ = this._isReady$.asObservable();
        this._currentChannel$ = new ReplaySubject();
        this.currentChannel$ = this._currentChannel$.asObservable();
        this.isInteropFdc3 = () => !!window.io;
        this.init();
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isReady) {
                logger.debug('fdc3 already connected');
                return;
            }
            try {
                yield fdc3Ready();
                this.isReady = true;
                this._isReady$.next(true);
                logger.debug('fdc3 connection success');
                yield this.getAndUpdateCurrentChannel();
                yield this.handleInteropChannelChanges();
            }
            catch (err) {
                logger.error('fdc3 connection error', err);
                throw err;
            }
        });
    }
    addIntentListeners(listeners) {
        listeners.forEach((handler, intent) => {
            const listener = addIntentListener(intent, handler);
            logger.debug(`fdc3 addIntentListener for '${intent}', ${listener}`);
        });
    }
    findIntent(intent, context) {
        logger.debug(`fdc3 findIntent '${intent}' '${context}'`);
        return findIntent(intent, context);
    }
    findIntentsByContext(context) {
        logger.debug(`fdc3 findIntentsByContext '${context}'`);
        return findIntentsByContext(context);
    }
    raiseIntentForContext(context) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug(`fdc3 raiseIntentForContext '${context}'`);
            return raiseIntentForContext(context);
        });
    }
    raiseIntent(context, intent) {
        if (intent) {
            logger.debug(`fdc3 raiseIntent '${intent}' '${context}'`);
            return raiseIntent(intent, context);
        }
        else {
            logger.warn(`fdc3 raiseIntent - no intent passed`);
        }
    }
    joinChannel(channelId) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug(`fdc3 joinChannel '${channelId}'`);
            if (((_a = this._currentChannel) === null || _a === void 0 ? void 0 : _a.id) === channelId) {
                return;
            }
            yield joinChannel(channelId);
            yield this.getAndUpdateCurrentChannel();
            return;
        });
    }
    getOrCreateChannel(channelId) {
        logger.debug(`fdc3 getOrCreateChannel '${channelId}'`);
        return getOrCreateChannel(channelId);
    }
    broadcastOnChannel(channelName, type, payload) {
        return __awaiter(this, void 0, void 0, function* () {
            const channel = yield getOrCreateChannel(channelName);
            this.broadcastMessageOnChannel(channel, payload, type);
        });
    }
    broadcastOnCurrentChannel(payload, type) {
        return __awaiter(this, void 0, void 0, function* () {
            const channel = yield getCurrentChannel();
            if (channel) {
                this.broadcastMessageOnChannel(channel, payload, type);
            }
        });
    }
    broadcastOnChannelEventHandler(channelName, type) {
        return __awaiter(this, void 0, void 0, function* () {
            return (e) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                // check for ag-grid-specific events, fall back to standard events
                const payload = ((_a = e.data) === null || _a === void 0 ? void 0 : _a.payload) || e.detail;
                yield this.broadcastOnChannel(channelName, type, payload);
            });
        });
    }
    addChannelListener(channelName, channelType, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            const channel = yield getOrCreateChannel(channelName);
            channel.addContextListener(channelType, callback);
            return channel;
        });
    }
    addSystemChannelListener(channelType, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            const channel = yield getCurrentChannel();
            channel.addContextListener(channelType, callback);
        });
    }
    getCurrentChannel() {
        return __awaiter(this, void 0, void 0, function* () {
            return getCurrentChannel();
        });
    }
    handleChannelCurrentContext(channel, contextType, listenerCallback) {
        return __awaiter(this, void 0, void 0, function* () {
            const currentContextValue = yield channel.getCurrentContext(contextType);
            if (currentContextValue) {
                listenerCallback(currentContextValue);
            }
        });
    }
    broadcastMessageOnChannel(channel, payload, type) {
        payload = stripOutBigInt(payload);
        const context = Object.assign({ type }, payload);
        channel.broadcast(context);
    }
    getAndUpdateCurrentChannel() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const channel = yield getCurrentChannel();
                if (channel) {
                    logger.debug(`fdc3 getCurrentChannel '${channel.id}'`);
                    this._currentChannel = channel;
                    this._currentChannel$.next(channel);
                }
            }
            catch (err) {
                logger.error('fdc3 getCurrentChannel error', err);
            }
        });
    }
    handleInteropChannelChanges() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isInteropFdc3()) {
                // Listen to changes from the io channel selector.
                // This can be done on the interop container window which is outside this app scope
                // Any changes made there will be emitted here and keep channel state in sync
                (_b = (_a = window.io) === null || _a === void 0 ? void 0 : _a.channels) === null || _b === void 0 ? void 0 : _b.onChanged((channelColor) => __awaiter(this, void 0, void 0, function* () {
                    var _c, _d;
                    if (((_d = (_c = this._currentChannel) === null || _c === void 0 ? void 0 : _c.displayMetadata) === null || _d === void 0 ? void 0 : _d.color) === channelColor) {
                        return;
                    }
                    logger.debug(`fdc3 interop.io channel changed '${channelColor}'`);
                    yield this.getAndUpdateCurrentChannel();
                }));
            }
        });
    }
}
__decorate([
    observable
], DefaultFDC3.prototype, "isReady", void 0);
/**
 * The DI token for the FDC3 interface.
 * @internal
 */
export const FDC3 = DI.createInterface((x) => x.singleton(DefaultFDC3));
